/* Integrates an ode.
 *
 *   23 October 1994
 *   Andrew D. Lewis
 *   California Institute of Technology
 */

#include<stdio.h>
#include<string.h>
#include "ode.h"
#define FALSE 0
#define TRUE 1

double parms[10];

main(argc,argv)
int argc;
char *argv[];
{
  double x[NDOF],xold[NDOF],t,tstep,tstart,tend,tstepuse;
  int i,ndofout,itest,jtest,iout[NDOF];
  char *dummy,*thefile,*whatfile,*output_fn,*input_fn,*log_fn,UsingStdIn;
  char *basename();
  FILE *output_fp,*input_fp,*log_fp;

/* Check to see if argument is present. */

  if (argc < 1) {
    printf("\nUsage: ode filename\n\n");
    return 0;
  }

  if (argc == 2) {
    UsingStdIn = FALSE;
  }
  else {
    UsingStdIn = TRUE;
  }

/* Get filenames setup. */

  if (!UsingStdIn) {
    thefile = (char *)malloc(strlen(argv[1]));
    strcpy(thefile,argv[1]);
    whatfile = basename(thefile,".dat");
    output_fn = (char *)malloc(strlen(argv[1])+4);
    input_fn = (char *)malloc(strlen(argv[1])+7);
    log_fn = (char *)malloc(strlen(argv[1])+7);
    strcpy(whatfile,argv[1]);
    strcpy(output_fn,whatfile);
    strcat(output_fn,".dat");
    strcpy(input_fn,whatfile);
    strcat(input_fn,".odeinp");
    strcpy(log_fn,whatfile);
    strcat(log_fn,".odelog");
  }

/* Input information from a file or from the screen. */

  if (!UsingStdIn) {

    itest = fopenerr(input_fn,-1);

    if (itest == -1) {

      jtest = fopenerr("odeinp.odeinp",1);
    
      if (jtest == -1) {
	printf("You will be asked to enter the input data by hand.\n");
	input_fp = stdin;
      }
      else {
	input_fp = fopen("odeinp.odeinp","r");
	printf("\nReading input information from odeinp.odeinp.\n");
      }
    }
    else {
      input_fp = fopen(input_fn,"r");
      printf("\nReading input information from %s.\n",input_fn);
    }
  }
  else {
    printf("You will be asked to enter the input data by hand.\n");
    input_fp = stdin;
  }

/* Open log file. */

  if (!UsingStdIn) {
    log_fp = fopen(log_fn,"w");
    fprintf(log_fp,"\n%s\n",log_fn);
    fclose(log_fp);
    dummy = (char *)malloc(strlen(log_fn)+8);
    strcpy(dummy,"date >> ");
    strcat(dummy,log_fn);
    system(dummy);
    log_fp = fopen(log_fn,"a");
  }

  if (!UsingStdIn) {
    fprintf(log_fp,"\nThe number of degrees of freedom is: %d\n",NDOF);
  }

/* Get parameters. */

  parmenter(parms,input_fp,log_fp,UsingStdIn);

/* Read initial conditions and print them to log file. */

  printf("\nEnter the initial conditions.\n");
  for (i = 0 ; i < NDOF ; i++) {
    fscanf(input_fp,"%lf",&xold[i]);
  }

  if (!UsingStdIn) {
    fprintf(log_fp,"\nThe initial conditions are:\n");
    for (i = 0 ; i < NDOF ; i++) {
      fprintf(log_fp,"%lf\n",xold[i]);
    }
  }

/* Initial condition hacks (for constraints, for example). */

  ichacks(xold);

/* Read the start time, finish time and time step and print them to the
   log file. */

  printf("Enter the start time.\n");
  fscanf(input_fp,"%lf",&tstart);
  if (!UsingStdIn) {
    fprintf(log_fp,"\nThe start time is: %lf\n",tstart);
  }

  printf("Enter the end time.\n");
  fscanf(input_fp,"%lf",&tend);
  if (!UsingStdIn) {
    fprintf(log_fp,"The end time is: %lf\n",tend);
  }

  printf("Enter the time step.\n");
  fscanf(input_fp,"%lf",&tstep);
  if (!UsingStdIn) {
    fprintf(log_fp,"The time step is: %lf\n",tstep);
  }

/* Allow the user to enter output specifics. */

  printf("\nHow many of the %d DOF do you want to output?\n",NDOF);
  fscanf(input_fp,"%d",&ndofout);

  printf("Enter %d integers indicating which values you want to output\n",ndofout);
  for (i = 0 ; i < ndofout ; i++) {
    fscanf(input_fp,"%d",&iout[i]);
  }

  if (!UsingStdIn) {
    fprintf(log_fp,"\nThe following degrees of freedom are output:\n");
    for (i = 0 ; i < ndofout ; i++) {
      fprintf(log_fp,"%d\n",iout[i]);
    }
    fprintf(log_fp,"\n");
  }
    
/* Close the input file and the log file and open the output file. */

  fclose(input_fp);
  if (!UsingStdIn) fclose(log_fp);

  if (!UsingStdIn) {
    output_fp = fopen(output_fn,"w");
  }
  else {
    output_fp = stdout;
  }

  if (!UsingStdIn) {
    printf("\nWriting output on %s\n",output_fn);
  }

  for (i = 0 ; i < ndofout ; i++) {
    fprintf(output_fp,"%lf  ",xold[iout[i]-1]);
  }
  fprintf(output_fp,"\n");

  t = tstart;

  if (tend > tstart) {
    while (t <= tend) {
      rk4(xold,t,tstep,x);
      for (i = 0 ; i < NDOF ; i++) {
	xold[i] = x[i];
      }
      for (i = 0 ; i < ndofout ; i++) {
	fprintf(output_fp,"%lf  ",x[iout[i]-1]);
      }
      fprintf(output_fp,"\n");
      t += tstep;
    }
  }
  if (tend < tstart) {
    tstepuse = -1.0 * tstep;
    while (t >= tend) {
      rk4(xold,t,tstepuse,x);
      for (i = 0 ; i < NDOF ; i++) {
	xold[i] = x[i];
      }
      for (i = 0 ; i < NDOF ; i++) {
	fprintf(output_fp,"%lf  ",x[i]);
      }
      fprintf(output_fp,"\n");
      t += tstepuse;
    }
  }

  fclose(output_fp);

  if (!UsingStdIn) {
    printf("\nOutput written on %s\n",output_fn);
    printf("\nInput information printed on %s\n\n",log_fn);
  }

  return 0;

}
